/*
 *  Copyright (c) 2016 UDBMS group
 *  *
 *  *  http://udbms.cs.helsinki.fi/
 *  *
 *  *  Author:  Chao Zhang   email: chao.z.zhang@helsinki.fi
 *  *
 *  * Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  * in the Software without restriction, including without limitation the rights
 *  * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  * copies of the Software, and to permit persons to whom the Software is
 *  * furnished to do so, subject to the following conditions:
 *  *
 *  * The above copyright notice and this permission notice shall be included in
 *  * all copies or substantial portions of the Software.
 *  *
 *  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 *  * SOFTWARE.
 *
 */

package com.datamodel;

import com.udbms.DataGenerator;

import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;
import javax.xml.bind.annotation.XmlType;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

/**
 * Created by chzhang on 7.10.2016.
 */
@XmlRootElement(name = "Invoice")
public class Invoice {
    public UUID Invoice_no = null;
    public UUID Order_no = null;
    public UUID Customer_no = null;
    public UUID RegUser_no = null;
    public String Invoice_date = "";
    public String Customer_name = "";
    public String Address = "";
    public double InvoiceAmount = 0;
    public double InvoiceVAT = 0;
    public double TotAmount = 0;
    public List<Invoiceline> invoicelines = null;

    public Invoice (){}

    public Invoice(Order order) throws ClassNotFoundException, IOException, InstantiationException {
            this.Invoice_no = UUID.randomUUID();
            this.Order_no = order.Order_no;
            this.Invoice_date = order.Orderdate;
            this.Customer_no = order.Customer_no;
            this.Customer_name = order.Customer_name;
            this.Address = DataGenerator.AddressName.getRandomName();
            this.InvoiceAmount = order.TotalSalesprice;
            this.InvoiceVAT = order.TotalVAT;
            this.TotAmount = order.TotAmount;
            this.invoicelines = new ArrayList<Invoiceline>();
            for(Orderline ol: order.orderlines)
            {
                Invoiceline il = new Invoiceline();
                il.Line_no = ol.Line_no;
                il.Product_no = ol.Product_no;
                il.Pcs = ol.Pcs;
                il.ListPrice = ol.ListPrice;
                il.DiscountPercentage = ol.DiscountPercentage;
                il.SalesPrice = ol.SalesPrice;
                il.OrderlineSalesPrice = ol.OrderlineSalesPrice;
                il.VATPercentage = ol.VATPercentage;
                il.OrderlineVAT = ol.OrderlineVAT;
                il.OrderlineTotAmount = ol.OrderlineTotAmount;
                invoicelines.add(il);
            }
        }
}
