/*
 *  Copyright (c) 2016 UDBMS group
 *  *
 *  *  http://udbms.cs.helsinki.fi/
 *  *
 *  *  Author:  Zhang Chao    email: chao.z.zhang@helsinki.fi
 *  *
 *  * Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  * in the Software without restriction, including without limitation the rights
 *  * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  * copies of the Software, and to permit persons to whom the Software is
 *  * furnished to do so, subject to the following conditions:
 *  *
 *  * The above copyright notice and this permission notice shall be included in
 *  * all copies or substantial portions of the Software.
 *  *
 *  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 *  * SOFTWARE.
 *
 */

package com.udbms;

/**
 * Created by chzhang on 5.10.2016.
 */
import java.io.*;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.*;
import com.datamodel.*;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.util.*;
import org.jgrapht.UndirectedGraph;
import org.jgrapht.graph.DefaultEdge;
import org.jgrapht.graph.SimpleGraph;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * Modified by Chao on 10.10.2016.
 */
public class DataGenerator {

    private static final String COMMA_DELIMITER = ",";

    private static final String NEW_LINE_SEPARATOR = "\n";

    private static Random rand = new Random();

    private static final int ratio = 5;

    private static final int MaxNumberofOrderLine = 5;

    private static final String VendorfileName = "./Unigen/Data/CSV_Vendors/";

    private static final String CustomerfileName = "./Unigen/Data/CSV_Customers/";

    private static final String RegUserfileName = "./Unigen/Data/CSV_RegUsers/";

    private static final String JSON_Product = "./Unigen/Data/JSON_Products/";

    private static final String JSON_Order = "./Unigen/Data/JSON_Orders/";

    private static final String XML_Invoice ="./Unigen/Data/XML_Invoices/";

    private static final String ReviewfileName = "./Unigen/Data/CSV_Reviews/";

    private static final String Social_Network = "./Unigen/Data/Social_Network/";

    private static final String Copyright="./Unigen/";

    private static final String VendorFILE_HEADER = "Vendor_no,Name,Vendor_no,DiscountPercentage";

    private static final String CustomerFILE_HEADER = "Customer_no,Name,Address,VATNo,CreditLimit,DiscountPercentage";

    private static final String RegUserFILE_HEADER = "RegUser_no,Name,Address";

    private static final String ReviewFILE_HEADER = "Person_Key,OrderLine_Key,Product_Key,Review";

    private static final String SocialNetwork_HEADER = "Person.id,Person.id,creationDate";

    public static final NameDictionary CompanyName= new NameDictionary("/data/CompanyName.csv");

    public static final NameDictionary AddressName= new NameDictionary("/data/Address.csv");

    public static final NameDictionary ProductName= new NameDictionary("/data/ProductName.csv");

    public static final NameDictionary FirstName= new NameDictionary("/data/FirstName.csv");

    public static final NameDictionary LastName= new NameDictionary("/data/LastName.csv");

    public static final OrganizationDictionary OrganizationDic = new OrganizationDictionary();

    public static final UniversitiesDictionary UniversitiesDic = new UniversitiesDictionary();

    public static void main(String[] args) throws IOException, JAXBException, InstantiationException,ClassNotFoundException {

        System.out.println("This is Multi-model data generator, Version 0.1\n" +
                           "by UDBMS group of University of Helsinki, Zhang Chao (chao.z.zhang@helsinki.fi), have fun!");
        System.out.println();

        // Control the data ratio by argument
        // Final arguments
        int[] intArgs = new int[8];
        // Get the Parameters
        if (args.length==0)
        args=new String[]{"0.1"};
        else if (args.length==1){
            // Round the double to one decimal place, and compute the final numbers of arguments
            DecimalFormat df = new DecimalFormat("#.#");
            DecimalFormatSymbols dfs = new DecimalFormatSymbols();
            dfs.setDecimalSeparator('.');
            df.setDecimalFormatSymbols(dfs);
            Double d = new Double(args[0]);
            d = Double.parseDouble(df.format(d));
            Double multiple = d*10;
            int mul = multiple.intValue();
            args = new String[]{"100","100","100","100","100","5","50","50"};
            for (int i=0;i<args.length;i++) {
                try {
                    if(i!=5)
                    intArgs[i] = Integer.parseInt(args[i])*mul*ratio;
                } catch (NumberFormatException e) {
                    System.err.println("Failed trying to parse a non-numeric argument, " + args[i]);
                }
            }
            intArgs[5] = MaxNumberofOrderLine;
        }

        // Create the data directory
        try {
            File v1=new File(VendorfileName);
            if(!v1.exists())
            v1.mkdirs();
            File c1=new File(CustomerfileName);
            if(!c1.exists())
            c1.mkdirs();
            File r1=new File(RegUserfileName);
            if(!r1.exists())
            r1.mkdirs();
            File r2=new File(ReviewfileName);
            if(!r2.exists())
            r2.mkdirs();
            File j1=new File(JSON_Product);
            if(!j1.exists())
            j1.mkdirs();
            File j2=new File(JSON_Order);
            if(!j2.exists())
            j2.mkdirs();
            File x1=new File(XML_Invoice);
            if(!x1.exists())
            x1.mkdirs();
            File sn=new File(Social_Network);
            if(!sn.exists())
            sn.mkdirs();
        } catch (Exception e) {
            e.printStackTrace();
        }

        Random random = new Random();

        //Generate the Vendors
        int vendorTimes = intArgs[0];
        List<Vendor> vendorList = new ArrayList<Vendor>();
        for (int i = 0; i < vendorTimes; i++) {
            Vendor V = null;
            try {
                V = new Vendor();
            } catch (IOException e) {
                e.printStackTrace();
            }
            vendorList.add(V);
        }
        System.out.println("The Vendor data is ready.");
        System.out.println("----------------");
        System.out.println("----------------");
        System.out.println("----------------");
        FileWriter fileWriter = null;
        BufferedWriter bufferedWriter=null;
        try {
            fileWriter = new FileWriter(VendorfileName+"Vendor.csv",true);
            bufferedWriter = new BufferedWriter(fileWriter);
            //Write the CSV file header
            bufferedWriter.write(VendorFILE_HEADER.toString());
            //Add a new line separator after the header
            bufferedWriter.write(NEW_LINE_SEPARATOR);
            //Write a new student object list to the CSV file
            for (Vendor vendor : vendorList) {
                bufferedWriter.write(String.valueOf(vendor.Vendor_no));
                bufferedWriter.write(COMMA_DELIMITER);
               bufferedWriter.write(vendor.Name);
               bufferedWriter.write(COMMA_DELIMITER);
               bufferedWriter.write(vendor.VATNo);
               bufferedWriter.write(COMMA_DELIMITER);
               bufferedWriter.write(String.valueOf(vendor.DiscountPercentage));
               bufferedWriter.write(NEW_LINE_SEPARATOR);
            }
        } catch (Exception e) {
            System.out.println("Error in CsvFileWriter !!!");
            e.printStackTrace();
        } finally {
            try {
                bufferedWriter.flush();
                bufferedWriter.close();
            } catch (IOException e) {
                System.out.println("Error while flushing/closing fileWriter !!!");
                e.printStackTrace();
            }

        }
        //Generate the Products
        int productTimes = intArgs[1];
        List<Product> productList = new ArrayList<Product>();
        int vendorindex = 0;
        for (int i = 0; i < productTimes; i++) {
            Product P = new Product();

            //Get a Random vendor
            vendorindex = random.nextInt(vendorList.size());
            P.Vendor_no = vendorList.get(vendorindex).Vendor_no;
            productList.add(P);
        }

        System.out.println("The Product data is ready.");
        System.out.println("----------------");
        System.out.println("----------------");
        System.out.println("----------------");

        //Generate the RegUsers
        int regUserTimes = intArgs[2];
        List<RegUser> reguserList = new ArrayList<RegUser>();
        for (int k = 1; k <= regUserTimes; k++) {
            RegUser R = null;
            try {
                R = new RegUser();
            } catch (IOException e) {
                e.printStackTrace();
            }
            reguserList.add(R);
        }

        System.out.println("The RegUser data is ready.");
        System.out.println("----------------");
        System.out.println("----------------");
        System.out.println("----------------");

        try {
            fileWriter = new FileWriter(RegUserfileName+"RegUsers.csv",true);
            bufferedWriter = new BufferedWriter(fileWriter);
            //Write the CSV file header
           bufferedWriter.write(RegUserFILE_HEADER.toString());
            //Add a new line separator after the header
           bufferedWriter.write(NEW_LINE_SEPARATOR);
            //Write a new student object list to the CSV file
            for (RegUser reguser : reguserList) {
               bufferedWriter.write(String.valueOf(reguser.RegUser_no));
               bufferedWriter.write(COMMA_DELIMITER);
               bufferedWriter.write(reguser.Name);
               bufferedWriter.write(COMMA_DELIMITER);
               bufferedWriter.write(reguser.Address);
               bufferedWriter.write(COMMA_DELIMITER);
               bufferedWriter.write(NEW_LINE_SEPARATOR);
            }
        } catch (Exception e) {
            System.out.println("Error in CsvFileWriter !!!");
            e.printStackTrace();
        } finally {
            try {
                bufferedWriter.flush();
                bufferedWriter.close();
            } catch (IOException e) {
                System.out.println("Error while flushing/closing fileWriter !!!");
                e.printStackTrace();
            }
        }

        //Generate the Customers
        int customerTimes = intArgs[3];
        List<Customer> CustomerList = new ArrayList<Customer>();
        for (int s = 1; s <= customerTimes; s++) {
            Customer R = null;
            R = new Customer();
            CustomerList.add(R);
        }

        System.out.println("The Customer data is ready.");
        System.out.println("----------------");
        System.out.println("----------------");
        System.out.println("----------------");

        try {
            fileWriter = new FileWriter(CustomerfileName+"Customer.csv",true);
            bufferedWriter = new BufferedWriter(fileWriter);
            //Write the CSV file header
           bufferedWriter.write(CustomerFILE_HEADER.toString());
            //Add a new line separator after the header
           bufferedWriter.write(NEW_LINE_SEPARATOR);
            //Write a new student object list to the CSV file
            for (Customer customer : CustomerList) {
               bufferedWriter.write(String.valueOf(customer.Customer_no));
               bufferedWriter.write(COMMA_DELIMITER);
               bufferedWriter.write(customer.Name);
               bufferedWriter.write(COMMA_DELIMITER);
               bufferedWriter.write(customer.Address);
               bufferedWriter.write(COMMA_DELIMITER);
               bufferedWriter.write(customer.VATNo);
               bufferedWriter.write(COMMA_DELIMITER);
               bufferedWriter.write(String.valueOf(customer.CreditLimit));
               bufferedWriter.write(COMMA_DELIMITER);
               bufferedWriter.write(String.valueOf(customer.DiscountPercentage));
               bufferedWriter.write(COMMA_DELIMITER);
               bufferedWriter.write(NEW_LINE_SEPARATOR);
            }
        } catch (Exception e) {
            System.out.println("Error in CsvFileWriter !!!");
            e.printStackTrace();
        } finally {
            try {
                bufferedWriter.flush();
                bufferedWriter.close();
            } catch (IOException e) {
                System.out.println("Error while flushing/closing fileWriter !!!");
                e.printStackTrace();
            }
        }
        //Generate the Orders
        int OrderCount = intArgs[4];
        int OrderLineCount = intArgs[5];
        List<Order> OrderList = new ArrayList<Order>();
        int Customerindex;
        int RegUserindex;
        int Productindex;
        int Orderlineindex;
        //Get a Random Dependency,half by customers
        for (int o = 1; o <= OrderCount/2; o++) {
            Order R = null;
            try {
                R = new Order(OrderLineCount);
            } catch (Exception e) {
                e.printStackTrace();
            }

            Customerindex = random.nextInt(CustomerList.size());
            R.Customer_no = CustomerList.get(Customerindex).Customer_no;
            R.Customer_name = CustomerList.get(Customerindex).Name;

            // Generate the orderline

            Orderlineindex = random.nextInt(OrderCount);

            //Orderlineindex can't be 0
            if (Orderlineindex == 0) {
                Orderlineindex++;
            }
            for (int j = 0; j < OrderLineCount; j++) {

                //Get a Random Dependency
                Productindex = random.nextInt(productList.size());
                R.orderlines.get(j).Product_no = productList.get(Productindex).Product_no;
                R.orderlines.get(j).Product_Name = productList.get(Productindex).Name;
                productList.get(Productindex).Price = R.orderlines.get(j).ListPrice;

            }
            OrderList.add(R);
        }

        //Get a Random Dependency,half by regusers
        for (int o = 1; o <= OrderCount/2; o++) {
            Order R = null;
            try {
                R = new Order(OrderLineCount);
            } catch (Exception e) {
                e.printStackTrace();
            }

            RegUserindex = random.nextInt(reguserList.size());
            R.RegUser_no = reguserList.get(RegUserindex).RegUser_no;

            // Generate the orderline

            Orderlineindex = random.nextInt(OrderCount);

            //Orderlineindex can't be 0
            if (Orderlineindex == 0) {
                Orderlineindex++;
            }
            for (int j = 0; j < OrderLineCount; j++) {

                //Get a Random Dependency
                Productindex = random.nextInt(productList.size());
                R.orderlines.get(j).Product_no = productList.get(Productindex).Product_no;
                R.orderlines.get(j).Product_Name = productList.get(Productindex).Name;
                productList.get(Productindex).Price = R.orderlines.get(j).ListPrice;

            }
            OrderList.add(R);
        }


        System.out.println("The Order data is ready.");
        System.out.println("----------------");
        System.out.println("----------------");
        System.out.println("----------------");


        int InvoiceCount = intArgs[6];
        ArrayList<UUID> InvoiceOfOrder = new ArrayList<UUID>();
        List<Order>InvoiceOfOrderList = new ArrayList<Order>();
        int Orderindex=0;
        while(InvoiceOfOrder.size()!=InvoiceCount) {
            Orderindex = random.nextInt(OrderList.size());
            if (!InvoiceOfOrder.contains(OrderList.get(Orderindex).Order_no)) {
                OrderList.get(Orderindex).Invoiced="Y";
                InvoiceOfOrder.add(OrderList.get(Orderindex).Order_no);
                InvoiceOfOrderList.add(OrderList.get(Orderindex));
            }
        }
        //Generate the Products
        for(Product product : productList){
            ProductobjectTojson(product);
        }

        //Generate the Orders
        for (Order order : OrderList) {
            OrderobjectTojson(order);
        }
        //Generate the Invoice
        for (Order order : InvoiceOfOrderList) {
            Invoice invoice = new Invoice(order);
            jaxbObjectToXML(invoice);
        }
        System.out.println("The Invoice data is ready.");
        System.out.println("----------------");
        System.out.println("----------------");
        System.out.println("----------------");

        /**
         *  Generate the graph data
         */

        // Find the person from same organization, person can be customer or reguser

        UndirectedGraph<UUID, DefaultEdge> SocialGraph = CreateSocialGraph(CustomerList,reguserList);

        System.out.println("The current edges of Social_Graph is " + SocialGraph.edgeSet().size());

        Map<String, List<Customer>> CustomerlistGroupBy = CustomerList.stream().collect(Collectors.groupingBy(w -> w.Organization));

        Map<String, List<RegUser>> RegUserlistGroupBy = reguserList.stream().collect(Collectors.groupingBy(w -> w.Organization));

        generate_Edges(SocialGraph,CustomerlistGroupBy,RegUserlistGroupBy);

        System.out.println("The current edges of Social_Graph after grouping by organization is " + SocialGraph.edgeSet().size());

        // Find the person from same University, person can be customer or reguser

        CustomerlistGroupBy = CustomerList.stream().collect(Collectors.groupingBy(w -> w.University));

        RegUserlistGroupBy = reguserList.stream().collect(Collectors.groupingBy(w -> w.University));

        generate_Edges(SocialGraph,CustomerlistGroupBy,RegUserlistGroupBy);

        System.out.println("The current edges of Social_Graph after grouping by university is " + SocialGraph.edgeSet().size());

        // Find the person randomly from customer or reguser

        generate_Edges(SocialGraph);

        System.out.println("The current edges of Social_Graph after randomly generating edges is " + SocialGraph.edgeSet().size());

        String vetexSource = "";

        String vetexTarget = "";

        // Generate the graph data to csv

        Set<DefaultEdge> se = SocialGraph.edgeSet();
        fileWriter = new FileWriter(Social_Network+"Social_Network.csv",true);
        bufferedWriter = new BufferedWriter(fileWriter);
       bufferedWriter.write(SocialNetwork_HEADER.toString());
       bufferedWriter.write(NEW_LINE_SEPARATOR);
        for (DefaultEdge de : SocialGraph.edgeSet()){
            vetexSource = de.toString().substring(1,37);
            vetexTarget = de.toString().substring(40,76);
            String creationDate = generate_Date();
            try {
               bufferedWriter.write(vetexSource);
               bufferedWriter.write(COMMA_DELIMITER);
               bufferedWriter.write(vetexTarget);
               bufferedWriter.write(COMMA_DELIMITER);
               bufferedWriter.write(creationDate);
               bufferedWriter.write(NEW_LINE_SEPARATOR);
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        try {
            bufferedWriter.flush();
            bufferedWriter.close();
            System.out.println("The Social_Network graph data is ready.");
            System.out.println("----------------");
            System.out.println("----------------");
            System.out.println("----------------");
        } catch (IOException e) {
            System.out.println("Error while flushing/closing fileWriter !!!");
            e.printStackTrace();
        }

        /*
         *  Generate the review(key/value) data
         */
        List<Review> RevList = new ArrayList<Review>();
        try {
            int ReviewTimes = intArgs[7];
            ReviewDictionary rd = new ReviewDictionary();
            String name;
            Order ord;
            Orderline ol;
            String value;
            // Link to Customers or Regusers
            // Distribution of positive and negative
            for (int r = 0; r < ReviewTimes/3; r++) {
                Review Rev = new Review();
                ord =  getRandomItem(OrderList);
                UUID randomKey1;
                if(ord.Customer_no==null)
                     randomKey1 = ord.RegUser_no;
                else
                     randomKey1=ord.Customer_no;
                // Set key
                Rev.setPersonkey(randomKey1);

                // Set value (positive sentence type : S+V+O)
                ol = getRandomItem(ord.orderlines);
                Rev.setOrderLineKey(ol.Line_no);
                Rev.setProductKey(ol.Product_no);
                name = ol.Product_Name;
                value = rd.getRandomSubject()+" "+rd.getRandomVerbObject_P()+" "+name+".";
                Rev.setReviewText(value);
                RevList.add(Rev);
            }
            // Set value (positive sentence type : S+V+C)
            for (int r = 0; r < ReviewTimes/3; r++) {
                Review Rev = new Review();
                ord =  getRandomItem(OrderList);
                UUID randomKey1;
                if(ord.Customer_no==null)
                    randomKey1 = ord.RegUser_no;
                else
                    randomKey1=ord.Customer_no;
                // Set key
                Rev.setPersonkey(randomKey1);
                ol = getRandomItem(ord.orderlines);
                Rev.setOrderLineKey(ol.Line_no);
                Rev.setProductKey(ol.Product_no);
                name = ol.Product_Name;
                value = name+" "+rd.getRandomVerbComplement_P();
                Rev.setReviewText(value);
                RevList.add(Rev);
            }
            // Set value (negative sentence type : S+V+C)
            for (int r = 0; r < ReviewTimes/6; r++) {
                Review Rev = new Review();
                ord =  getRandomItem(OrderList);
                UUID randomKey1;
                if(ord.Customer_no==null)
                    randomKey1 = ord.RegUser_no;
                else
                    randomKey1=ord.Customer_no;
                // Set key
                Rev.setPersonkey(randomKey1);
                ol = getRandomItem(ord.orderlines);
                Rev.setOrderLineKey(ol.Line_no);
                Rev.setProductKey(ol.Product_no);
                name = ol.Product_Name;
                value = name+" "+rd.getRandomVerbComplement_N();
                Rev.setReviewText(value);
                RevList.add(Rev);
            }
            // Set value (negative sentence type : S+V+O)
            for (int r = 0; r < ReviewTimes/6; r++) {
                Review Rev = new Review();
                ord =  getRandomItem(OrderList);
                UUID randomKey1;
                if(ord.Customer_no==null)
                    randomKey1 = ord.RegUser_no;
                else
                    randomKey1=ord.Customer_no;
                // Set key
                Rev.setPersonkey(randomKey1);
                ol = getRandomItem(ord.orderlines);
                Rev.setOrderLineKey(ol.Line_no);
                Rev.setProductKey(ol.Product_no);
                name = ol.Product_Name;
                value = rd.getRandomSubject()+" "+rd.getRandomVerbObject_N()+" "+name+".";
                Rev.setReviewText(value);
                RevList.add(Rev);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        try {
            fileWriter = new FileWriter(ReviewfileName+"Review.csv",true);
            bufferedWriter = new BufferedWriter(fileWriter);
            //Write the CSV file header
           bufferedWriter.write(ReviewFILE_HEADER.toString());
            //Add a new line separator after the header
           bufferedWriter.write(NEW_LINE_SEPARATOR);
            //Write a new student object list to the CSV file
            for (Review rev : RevList) {
               bufferedWriter.write(String.valueOf(rev.getPersonkey()));
               bufferedWriter.write(COMMA_DELIMITER);
               bufferedWriter.write(String.valueOf(rev.getOrderLineKey()));
               bufferedWriter.write(COMMA_DELIMITER);
               bufferedWriter.write(String.valueOf(rev.getProductKey()));
               bufferedWriter.write(COMMA_DELIMITER);
               bufferedWriter.write(rev.getReviewText());
               bufferedWriter.write(NEW_LINE_SEPARATOR);
            }
        } catch (Exception e) {
            System.out.println("Error in CsvFileWriter !!!");
            e.printStackTrace();
        } finally {
            try {
                bufferedWriter.flush();
                bufferedWriter.close();
            } catch (IOException e) {
                System.out.println("Error while flushing/closing fileWriter !!!");
                e.printStackTrace();
            }
        }


        System.out.println("The Review data is ready.");
        System.out.println("----------------");
        System.out.println("----------------");
        System.out.println("----------------");
        System.out.println("All Data is ready!!");


        File cr=new File(Copyright+"Copyright.txt");
        if(!cr.exists()) {
            //Copyright
            try {
                fileWriter = new FileWriter(Copyright + "Copyright.txt", true);
                bufferedWriter = new BufferedWriter(fileWriter);
                bufferedWriter.append("Copyright (c) 2016 UDBMS group,http://udbms.cs.helsinki.fi/");
                bufferedWriter.append(NEW_LINE_SEPARATOR);
                bufferedWriter.append(" Author: Zhang Chao email: chao.z.zhang@helsinki.fi");
                bufferedWriter.newLine();
                bufferedWriter.newLine();
                bufferedWriter.newLine();
                bufferedWriter.append(" Permission is hereby granted, free of charge, to any person obtaining a copy\n" +
                        " of this software and associated documentation files (the \"Software\"), to deal\n" +
                        " in the Software without restriction, including without limitation the rights\n" +
                        " to use, copy, modify, merge, publish, distribute, sublicense, and/or sell\n" +
                        " copies of the Software, and to permit persons to whom the Software is\n" +
                        " furnished to do so, subject to the following conditions:\n" +
                        " The above copyright notice and this permission notice shall be included in\n" +
                        " all copies or substantial portions of the Software.\n" +
                        " THE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\n" +
                        " IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\n" +
                        " FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\n" +
                        " AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\n" +
                        " LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\n" +
                        " OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\n" +
                        " SOFTWARE.\n");
            } catch (Exception e) {
                System.out.println("Error in CsvFileWriter !!!");
                e.printStackTrace();
            } finally {
                try {
                    bufferedWriter.flush();
                    bufferedWriter.close();
                } catch (IOException e) {
                    System.out.println("Error while flushing/closing fileWriter !!!");
                    e.printStackTrace();
                }
            }

        }

        }

    public static String generate_Text(String filename) throws FileNotFoundException {
        Resource res=new Resource();
        filename ="/data/" + filename;
        String Text = "";
        BufferedReader fr= null;
        try {
            fr = res.getResource(filename);
        } catch (IOException e) {
            e.printStackTrace();
        }
        LineNumberReader reader  = new LineNumberReader(fr);
        int linecount = 0;
        String lineRead = "";
        try {
            while ((lineRead = reader.readLine()) != null) {}
        } catch (IOException e) {
            e.printStackTrace();
        }

        linecount = reader.getLineNumber();
        try {
            reader.close();
        } catch (IOException e) {
            System.out.println("2");
            e.printStackTrace();
        }
        Random randomGenerator = new Random();
        int lineNumber = randomGenerator.nextInt(linecount);
        try (Stream<String> lines = Files.lines(Paths.get(filename))) {
            Text = lines.skip(lineNumber).findFirst().get();
            // Charset.forName("ASCII");
        } catch (IOException e) {
            e.printStackTrace();
        }
        return Text;

    }

    public static String generate_Date() {
        String date="";
        int yearBegin=2015;
        int yearEnd=2016-yearBegin;
        date=""+(1 + (int)(Math.random() * 31)+"/"+(1 + (int)(Math.random() * 12)+"/"+(yearBegin + (int)(Math.random() * yearEnd))));
        return date;
    }

    public static String generate_VATno(String VATno) {
        char[] Cvalues = {'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z'};
        char[] Nvalues = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9'};
        String vatno = "";
        Random random = new Random((new Date()).getTime());
        for (int i = 0; i < 2; i++) {
            int idx = random.nextInt(Cvalues.length);
            vatno += Cvalues[idx];
        }
        for (int i = 0; i < 8; i++) {
            int idx = random.nextInt(Nvalues.length);
            vatno += Nvalues[idx];
        }
        return vatno;
    }

    public static int generate_Integer() {
        Random randomGenerator = new Random();
        int randomInt=randomGenerator.nextInt(100);
        while(randomInt==0) {
            randomInt = randomGenerator.nextInt(100);
        }
        return randomInt;
    }

    public static int generate_Price(int price) {
        Random r = new Random();
        int p= r.nextInt(price);
        return p;
    }

    public static void ProductobjectTojson(Product product){
        UUID fileno = product.Product_no;
        String filename = "Product" + fileno.toString() + ".json";
        String dgdir = JSON_Product;
        String filepath = dgdir + filename;
        ObjectMapper mapper = new ObjectMapper();
        try {
            mapper.writeValue(new File(filepath), product);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void jaxbObjectToXML(Invoice inv) {

        try {
            JAXBContext context = JAXBContext.newInstance(Invoice.class);
            Marshaller m = context.createMarshaller();
            m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
            UUID fileno = inv.Invoice_no;
            String dgdir =XML_Invoice;
            String filename = dgdir + "Invoice" + fileno.toString()+".xml";
            m.marshal(inv, new File(filename));
        } catch (JAXBException e) {
            e.printStackTrace();
        }
    }

    public static void OrderobjectTojson(Order order){
        UUID fileno = order.Order_no;
        String filename = "Order" + fileno.toString() + ".json";
        String dgdir = JSON_Order;
        String filepath = dgdir + filename;
        ObjectMapper mapper = new ObjectMapper();
        try {
            mapper.writeValue(new File(filepath), order);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static double generate_Percentage(double perc) {
        double d = Math.random() * 100;
        d = Math.round(d);
        return d;

    }

    public static double round(double value, int places) {
        if (places < 0) throw new IllegalArgumentException();

        long factor = (long) Math.pow(10, places);
        value = value * factor;
        long tmp = Math.round(value);
        return (double) tmp / factor;
    }

    private static <T> T getRandomItem(List<T> list) {

        return list.get(rand.nextInt(list.size()));
    }

    /**
     * Create a  graph based on Customer and RegUser objects.
     *
     * @return a graph based on Customer and RegUser objects.
     */
    private static UndirectedGraph<UUID, DefaultEdge> CreateSocialGraph(List<Customer> customerlist, List<RegUser> reguserlist)
    {
        UndirectedGraph<UUID, DefaultEdge> SGraph = new SimpleGraph<>(DefaultEdge.class);
        for(Customer cust:customerlist){
            SGraph.addVertex(cust.Customer_no);
        }
        for(RegUser regUser:reguserlist){
            SGraph.addVertex(regUser.RegUser_no);
        }
        return SGraph;
    }

    /**
     * Generate the edges of graph based on Customer and RegUser objects.
     *
     * @return a graph based on Customer and RegUser objects.
     */
    private static UndirectedGraph<UUID, DefaultEdge> generate_Edges(UndirectedGraph<UUID, DefaultEdge> Graph,Map<String, List<Customer>> CustomerGroup,  Map<String, List<RegUser>>  ReguserGroup){

        Set<String> groupedCustomersKeySet = CustomerGroup.keySet();

        Set<String> groupedRegUsersKeySet = ReguserGroup.keySet();

        for(String org: groupedCustomersKeySet)
        {
            if(groupedRegUsersKeySet.contains(org)) {

                List<Customer> custlist = CustomerGroup.get(org);

                List<RegUser> reglist = ReguserGroup.get(org);

                List<UUID> similarlist = new ArrayList<UUID>();

                for (RegUser regUser : reglist) {
                    similarlist.add(regUser.RegUser_no);
                }
                for (Customer Cust : custlist) {
                    similarlist.add(Cust.Customer_no);
                }

                // Add no repeat edges to Graph

                for(UUID key1 : similarlist){
                    for(UUID key2 : similarlist){
                        if(key1==key2){
                            continue;
                        }
                        if(Graph.containsEdge(key1,key2)){
                            continue;
                        }
                        Graph.addEdge(key1,key2);
                    }
                }
            }
        }
        return  Graph;
    }

    /**
     * Generate the edges of graph randomly, size equals to current edges
     *
     * @return a graph.
     */
    private static UndirectedGraph<UUID, DefaultEdge> generate_Edges(UndirectedGraph<UUID, DefaultEdge> Graph){

        int EdgeSize =Graph.edgeSet().size();

        UUID randomKey1;

        UUID randomKey2;

        Random randomGenerator = new Random();

        List<UUID> KeyList = new ArrayList<UUID>(Graph.vertexSet());

        for(int i=0;i<EdgeSize;i++){
            randomKey1 = KeyList.get(randomGenerator.nextInt(KeyList.size()));
            randomKey2 = KeyList.get(randomGenerator.nextInt(KeyList.size()));
            if(randomKey1!= randomKey2){
                if (!Graph.containsEdge(randomKey1,randomKey2)) {
                    Graph.addEdge(randomKey1,randomKey2);
                }
            }
        }
        return  Graph;
    }
}
