/*
 *  Copyright (c) 2016 UDBMS group
 *  *
 *  *  http://udbms.cs.helsinki.fi/
 *  *
 *  *  Author:  Chao Zhang   email: chao.z.zhang@helsinki.fi
 *  *
 *  * Permission is hereby granted, free of charge, to any person obtaining a copy
 *  of this software and associated documentation files (the "Software"), to deal
 *  * in the Software without restriction, including without limitation the rights
 *  * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 *  * copies of the Software, and to permit persons to whom the Software is
 *  * furnished to do so, subject to the following conditions:
 *  *
 *  * The above copyright notice and this permission notice shall be included in
 *  * all copies or substantial portions of the Software.
 *  *
 *  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 *  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 *  * SOFTWARE.
 *
 */

package com.util;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.*;

/**
 * Created by chzhang on 17.10.2016.
 */
public class UniversitiesDictionary {

    private static final String SEPARATOR = "  ";

    private static final String fileName = "/data/universities.txt";

    private static Random rand = new Random();

    private static Map<String,List<String>>  UniversitiesByCountry = new HashMap<>();

    private static Set<String> KeySet;

    public UniversitiesDictionary(){
        load(fileName);
    }

    private void load(String fileName){
        try {
            BufferedReader dicAllInstitutes = new BufferedReader(
                    new InputStreamReader(UniversitiesDictionary.class.getResourceAsStream(fileName), "UTF-8"));

            String line;
            while ((line = dicAllInstitutes.readLine()) != null) {
                String data[] = line.split(SEPARATOR);
                String countryName = data[0];
                String UniversityName = data[1];
                String cityName = data[2];
                if(!UniversitiesByCountry.keySet().contains(countryName)) {
                    List<String> NewList = new ArrayList<String>();
                    NewList.add(UniversityName);
                    UniversitiesByCountry.put(countryName,NewList);
                }
                else{
                    UniversitiesByCountry.get(countryName).add(UniversityName);
                }

            }
            KeySet = UniversitiesByCountry.keySet();
            dicAllInstitutes.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public  String getRandomKey() {
        List<String> keysAsArray = new ArrayList<String>(KeySet);
        return keysAsArray.get(rand.nextInt(keysAsArray.size()));
    }

    public  String getRandomItem() {
        String key = getRandomKey();
        List<String> keysAsArray = UniversitiesByCountry.get(key);
        return keysAsArray.get(rand.nextInt(keysAsArray.size()));
    }

    public String getKey(Integer value){
        for(String key : UniversitiesByCountry.keySet()){
            if(UniversitiesByCountry.get(key).equals(value)){
                return key; //return the first found
            }
        }
        return null;
    }
}
